# # SPDX-FileCopyrightText: 2023 Karlsruher Institut für Technologie
# #
# # SPDX-License-Identifier: CC0-1.0

# import sys

# from tds2stac.main_app import Catalog_summary

# sys.path.append("../")


# def test_catalog_summary_first_scenario_zero():
#     Catalog_summary(
#         "https://thredds.imk-ifu.kit.edu/thredds/catalog/regclim/regional/seas5_bc/forecast_measures/catalog.html",
#         depth_number=0,
#     )


# def test_catalog_summary_first_scenario_gt_zero():
#     Catalog_summary(
#         "https://thredds.imk-ifu.kit.edu/thredds/catalog/regclim/regional/seas5_bc/forecast_measures/catalog.html",
#         depth_number=1,
#     )


# def test_catalog_summary_second_scenario_zero():
#     Catalog_summary(
#         "https://thredds.imk-ifu.kit.edu/thredds/catalog/catalogues/sensor_catalog_ext.html",
#         depth_number=0,
#     )


# def test_catalog_summary_second_scenario_gt_zero():
#     Catalog_summary(
#         "https://thredds.imk-ifu.kit.edu/thredds/catalog/catalogues/sensor_catalog_ext.html",
#         depth_number=1,
#     )


# def test_catalog_summary_third_scenario_zero():
#     Catalog_summary(
#         "https://thredds.imk-ifu.kit.edu/thredds/catalog/regclim/raster/global/chirps/catalog.html",
#         depth_number=0,
#     )


# def test_catalog_summary_third_scenario_gt_zero():
#     Catalog_summary(
#         "https://thredds.imk-ifu.kit.edu/thredds/catalog/regclim/raster/global/chirps/catalog.html",
#         depth_number=1,
#     )


# def test_catalog_summary_fourth_scenario():
#     Catalog_summary(
#         "https://thredds.atmohub.kit.edu/thredds/catalog/snowfogs/catalog.html",
#         depth_number=0,
#     )


# def test_catalog_summary_fifth_scenario():
#     Catalog_summary(
#         "https://thredds.imk-ifu.kit.edu/thredds/catalog/climate/raster/global/chelsa/v1.2/catalog.html",
#         depth_number=1,
#     )


# def test_catalog_summary_sixth_scenario():
#     Catalog_summary(
#         "https://thredds.imk-ifu.kit.edu/thredds/catalog/regclim/raster/global/era5/sfc/single/daily/catalog.html?dataset=era5_sfc_0.25_single/daily/ERA5_daily_sp_1979.nc",
#         depth_number=0,
#     )


# def test_catalog_summary_seventh_scenario():
#     Catalog_summary(
#         "https://thredds.imk-ifu.kit.edu/thredds/catalog/catalogues/climate_catalog_ext.html?dataset=gleam_v3.5a_daily_aggregated",
#         depth_number=1,
#     )


# def test_catalog_summary_eighth_scenario_zero():
#     Catalog_summary(
#         "https://thredds.imk-ifu.kit.edu/thredds/catalog/catalogues/transfer.html",
#         depth_number=0,
#     )


# def test_catalog_summary_eighth_scenario_gt_zero():
#     Catalog_summary(
#         "https://thredds.imk-ifu.kit.edu/thredds/catalog/catalogues/transfer.html",
#         depth_number=1,
#     )


# def test_catalog_summary_ninth_scenario_zero():
#     Catalog_summary(
#         "https://thredds.imk-ifu.kit.edu/thredds/catalog/regclim/raster/global/hydrogfd/v3.0/catalog.html",
#         depth_number=0,
#     )


# def test_catalog_summary_ninth_scenario_gt_zero():
#     Catalog_summary(
#         "https://thredds.imk-ifu.kit.edu/thredds/catalog/regclim/raster/global/hydrogfd/v3.0/catalog.html",
#         depth_number=1,
#     )
